<?php

/**
 * @file
 * Definition of Drupal\delta_filter\Plugin\views\filter\DeltaFilter.
 */

namespace Drupal\delta_filter\Plugin\views\filter;

use Drupal\views\Plugin\views\field\NumericField;
use Drupal\views\Plugin\views\filter\FilterPluginBase;
use Drupal\Core\Form\FormStateInterface;


/**
 * Filters multivalue fields to have the same delta.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("delta_filter")
 */

class DeltaFilter extends FilterPluginBase {

  /**
   * {@inheritdoc}
   */
  public function canExpose() {
    return FALSE;
  }

  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['fields'] = ['default' => []];

    return $options;
  }

  public function operators() {
    // no operators needed
    $operators = [];
  }

  /**
   * Display the filter on the administrative summary
   */
  public function adminSummary() {
    return '= multiple fields';
  }


  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);
    $this->view->initStyle();

    // Allow to choose all fields as possible
    if ($this->view->style_plugin->usesFields()) {
      $options = [];
      foreach ($this->view->display_handler->getHandlers('field') as $name => $field) {
        // Only allow delta fields.
        if ($field->realField === 'delta') {
          $options[$field->table] = $field->table . ' (delta)';
        }
      }
      if ($options) {
        $form['fields'] = [
          '#type' => 'select',
          '#title' => $this->t('Choose fields to filter on having the same delta'),
          '#description' => $this->t("This filter compares delta's for multivalue fields."),
          '#multiple' => TRUE,
          '#options' => $options,
          '#default_value' => $this->options['fields'],
        ];
//        dump($options);
      }
      else {
        $form_state->setErrorByName('', $this->t('You have to add some fields to be able to use this filter.'));
      }
    }
  }

  /**
   * Add this filter to the query.
   * The number of delta's is based on the selection of fields in the form options
   */
  public function query() {
    $this->ensureMyTable();
    $deltafields = [];
    // build array of fields needed for SQL
    foreach ($this->view->field as $fieldname => $field) {
      if ($field instanceof NumericField && $field->realField === 'delta' && in_array($field->table, $this->options['fields'])) {
        $deltafields[] = [
          'alias' => $field->table,
          'real' => $field->realField,
          ];
      }
    }

    // if deltafields does not have multiple fields abondon
    if (count($deltafields) < 2) {
      return;
    }

    // define filter group
    $this->query->setWhereGroup('AND', 'DeltaFilters');

    // Build pieces of SQL based on number of deltafields.
    $i = 0;
    $j = 1;
    do {
      $snippet[$i] = $deltafields[$i]['alias'] . '.' . $deltafields[$i]['real'] . ' ' . '=' . ' ' . $deltafields[$j]['alias'] . '.' . $deltafields[$j]['real'];
      $this->query->addWhereExpression('DeltaFilters', $snippet[$i]);
      $i++;
      $j++;
    } while ($j < count($deltafields));
  }
}
